PID_COL = "PID"
TREATMENT_COL = "treatment"
OUTCOME_COL = "outcome"
PS_COL = "ps"
PROBAS_T1_COL = "probas_t1"  # probability of outcome when treated
PROBAS_T0_COL = "probas_t0"  # probability of outcome when not treated
PROBAS_COL = "probas"  # probability of outcome under given treatment


OUTCOME_COL = "Y"
OUTCOME_CF_COL = "Y_cf"

TREATED_PID_COL = "treated_pid_col"
CONTROL_PID_COL = "control_pid_col"
DISTANCE_COL = "distance_col"

EFFECT = "effect"
EFFECT_treated = "effect_1"
EFFECT_untreated = "effect_0"

INITIAL_EFFECT = "initial_effect"
INITIAL_EFFECT_treated = "initial_effect_1"
INITIAL_EFFECT_untreated = "initial_effect_0"
ADJUSTMENT_treated = "adjustment_1"
ADJUSTMENT_untreated = "adjustment_0"

STD_ERR = "std_err"
CI95_LOWER = "CI95_lower"
CI95_UPPER = "CI95_upper"
